package ca.canadapost.cpcdp.shipping;


import ca.canadapost.cpcdp.shipping.generated.shipment.ReferencesType;
import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;
import ca.canadapost.cpcdp.shipping.generated.shipment.ParcelCharacteristicsType.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;


import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.GregorianCalendar; 
import java.util.Properties;

import java.math.*;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

/**
 * Sample code for the CreateShipment Canada Post service.
 * <p>
 * The CreateShipment service is used to create a new shipping item, to 
 * request the generation of a softcopy image of shipping labels, and to provide 
 * links to these shipping labels and other information associated with the 
 * shipping item.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for CreateShipment Request.  <br>
 *  USAGE:<pre>
 *        CreateShipmentClient client = new CreateShipmentClient(username, password);
 *        ClientResponse response = client.createShipment(shipment);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class CreateShipmentClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/shipment";
    	
		// Create CreateShipment XML Request Object
        Shipment shipment = new Shipment();
        
        JAXBElement<String> groupId = new JAXBElement<String>(new QName("http://www.canadapost.ca/ws/shipment-v8", "group-id"), String.class, "4326432");
		JAXBElement<Boolean> transmitShipment = new JAXBElement<Boolean>(new QName("http://www.canadapost.ca/ws/shipment-v8", "transmit-shipment"), Boolean.class, Boolean.TRUE);
        
		//1. regular shipment
		shipment.setGroupIdOrTransmitShipment(groupId);

        //2. no manifest shipment
        //shipment.setGroupIdOrTransmitShipment(transmitShipment);
		
		
		shipment.setRequestedShippingPoint("H2B1A0");
		shipment.setCpcPickupIndicator(true);
		GregorianCalendar mailingDate = new GregorianCalendar(); 
		mailingDate.set(2015, 10, 28); 
		XMLGregorianCalendar XmlMailingDate = null;
		try {
			XmlMailingDate = DatatypeFactory.newInstance().newXMLGregorianCalendar(mailingDate);
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace(System.out);
		} 
		shipment.setExpectedMailingDate(XmlMailingDate);
		
		AddressDetailsType senderAddress = new AddressDetailsType();
		senderAddress.setAddressLine1("502 MAIN ST N");
		senderAddress.setCity("MONTREAL");
		senderAddress.setProvState("QC");
		senderAddress.setCountryCode("CA");
		senderAddress.setPostalZipCode("H2B1A0");
		
		SenderType sender = new SenderType();
		sender.setName("Bulma");
		sender.setCompany("Capsule Corp.");
		sender.setContactPhone("1 (514) 820 5879");
		sender.setAddressDetails(senderAddress);
		
		DestinationAddressDetailsType destinationAddress = new DestinationAddressDetailsType();
		destinationAddress.setAddressLine1("123 Postal Drive");
		destinationAddress.setCity("Ottawa");
		destinationAddress.setProvState("ON");
		destinationAddress.setCountryCode("CA");
		destinationAddress.setPostalZipCode("K1P5Z9");
		
		DestinationType destination = new DestinationType();
		destination.setName("John Doe");
		destination.setCompany("ACME Corp.");
		destination.setAddressDetails(destinationAddress);
		
		OptionType option1 = new OptionType();
		option1.setOptionCode("DC");
		
		DeliverySpecType.Options options = new DeliverySpecType.Options();
		options.getOptions().add(option1);

		Dimensions parcelDimension = new Dimensions();
		parcelDimension.setHeight(new BigDecimal(6));
		parcelDimension.setLength(new BigDecimal(12));
		parcelDimension.setWidth(new BigDecimal(9));

		ParcelCharacteristicsType characteristics = new ParcelCharacteristicsType();
		characteristics.setWeight(new BigDecimal(5));		
		characteristics.setDimensions(parcelDimension);

		NotificationType notification = new NotificationType();;
		notification.setEmail("ryuko.saito@kubere.com");
		notification.setOnDelivery(true);
		notification.setOnException(false);
		notification.setOnShipment(true);
		
		PrintPreferencesType printPreferences = new PrintPreferencesType();
		printPreferences.setOutputFormat("8.5x11");
		
		PreferencesType preferences = new PreferencesType();
		preferences.setShowInsuredValue(true);
		preferences.setShowPackingInstructions(false);
		preferences.setShowPostageRate(true);
		
		ReferencesType references = new ReferencesType();
		references.setCostCentre("costCentre");
		references.setCustomerRef1("custRef1");
		references.setCustomerRef2("custRef2");
		
		SettlementInfoType settlementInfo = new SettlementInfoType();
		settlementInfo.setContractId("0042708517");
		settlementInfo.setIntendedMethodOfPayment("Account");
		
		DeliverySpecType deliverySpec = new DeliverySpecType();
		deliverySpec.setServiceCode("DOM.EP");
		deliverySpec.setSender(sender);
		deliverySpec.setDestination(destination);
		deliverySpec.setOptions(options);
		deliverySpec.setParcelCharacteristics(characteristics);
		deliverySpec.setNotification(notification);
		deliverySpec.setPrintPreferences(printPreferences);
		deliverySpec.setPreferences(preferences);
		deliverySpec.setReferences(references);
		deliverySpec.setSettlementInfo(settlementInfo);
		
		shipment.setDeliverySpec(deliverySpec);
    	
		// Execute CreateShipment Request
        CreateShipmentClient myClient = new CreateShipmentClient(username, password);
        ClientResponse resp = myClient.createShipment(shipment);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(ShipmentInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateShipmentInfo schema.
            if (entity instanceof ShipmentInfo) {
            	ShipmentInfo shipmentInfo = (ShipmentInfo) entity;
            	System.out.println("Shipment ID: " + shipmentInfo.getShipmentId());                
                for (Iterator<Link> iter = shipmentInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CreateShipmentClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createShipment(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.shipment-v8+xml")
        		.header("Content-Type", "application/vnd.cpc.shipment-v8+xml")
        		.header("Source-App", "EXT")
        		.header("Consumer-Id", "1")
        		.acceptLanguage("en-CA")
        		.post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
