package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetShipmentDetails Canada Post service.
 * <p> 
 * The GetShipmentDetails service is used to retrieve the shipment details in XML 
 * format for a shipment created by a prior "create shipment" call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetShipmentDetails Request.  <br>
 *  USAGE:<pre>
 *        GetShipmentDetailsClient client = new GetShipmentDetailsClient(username, password);
 *        ClientResponse response = client.getShipmentDetails();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetShipmentDetailsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// Use link from CreateShipment
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/shipment/340531309186521749/details";		
		
		// Execute GetShipmentDetails Request
        GetShipmentDetailsClient myClient = new GetShipmentDetailsClient(username, password);
        ClientResponse resp = myClient.getShipmentDetails();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(ShipmentDetails.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches ShipmentDetails schema.
            if (entity instanceof ShipmentDetails) {
            	ShipmentDetails shipmentDetails = (ShipmentDetails) entity;
            	System.out.println("Tracking Pin: " + shipmentDetails.getTrackingPin());                
            	
            	if(shipmentDetails.getShipmentDetail().getGroupIdOrTransmitShipment().getDeclaredType() == Boolean.class){
            		System.out.println("No manifest shipment: " + shipmentDetails.getShipmentDetail().getGroupIdOrTransmitShipment().getValue());
            	}
            	else{
            		System.out.println("Group Id: " + shipmentDetails.getShipmentDetail().getGroupIdOrTransmitShipment().getValue());
            	}
            	
            	System.out.println("Sender Postal Code: " + shipmentDetails.getShipmentDetail().getDeliverySpec().getSender().getAddressDetails().getPostalZipCode());                
            	System.out.println("Destination Postal Code: " + shipmentDetails.getShipmentDetail().getDeliverySpec().getDestination().getAddressDetails().getPostalZipCode());                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetShipmentDetailsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getShipmentDetails() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.shipment-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
