package ca.canadapost.cpcdp.shipping;


import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.manifest.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the TransmitShipments Canada Post service.
 * <p>
 * The TransmitShipments service is used to specify shipments to be included in a manifest. 
 * Inclusion in a manifest is specified by group. Specific shipments may be excluded if 
 * desired.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for TransmitShipments Request.  <br>
 *  USAGE:<pre>
 *        TransmitShipmentsClient client = new TransmitShipmentsClient(username, password);
 *        ClientResponse response = client.transmitShipments(transmit);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class TransmitShipmentsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/manifest";
    	
		// Create TransmitShipments XML Request Object
        TransmitSet transmit = new TransmitSet();
        
        GroupIDListType groupIdList = new GroupIDListType();
        groupIdList.getGroupIds().add("4326432");
        
        AddressDetailsType addressDetails = new AddressDetailsType();

        addressDetails.setAddressLine1("1230 Tako RD."	);
        addressDetails.setCity("Ottawa");
        addressDetails.setProvState("ON");
        addressDetails.setPostalZipCode("K2B8J6");
        addressDetails.setCountryCode("CA");

        ManifestAddressType manifestAddress = new ManifestAddressType();
        manifestAddress.setManifestCompany("MajorShop");
        manifestAddress.setPhoneNumber("514 829 5879");
        manifestAddress.setAddressDetails(addressDetails);
        
        transmit.setGroupIds(groupIdList);
        transmit.setRequestedShippingPoint("K1K4T3");
        transmit.setCpcPickupIndicator(true);
        transmit.setDetailedManifests(true);
        transmit.setMethodOfPayment("Account");
        transmit.setManifestAddress(manifestAddress);
            	
		// Execute TransmitShipments Request
        TransmitShipmentsClient myClient = new TransmitShipmentsClient(username, password);
        ClientResponse resp = myClient.transmitShipments(transmit);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Manifests.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Manifests schema.
            if (entity instanceof Manifests) {
            	Manifests manifests = (Manifests) entity;
                for (Iterator<Link> iter = manifests.getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public TransmitShipmentsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse transmitShipments(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.manifest-v8+xml").header("Content-Type", "application/vnd.cpc.manifest-v8+xml").acceptLanguage("en-CA").post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
